//////////////////////////////////////////
// FSL Fast I/O Controller
//
// Copyright 2004-2005, Claudio Lanconelli
// http://www.lancos.com
//////////////////////////////////////////

#include "xstatus.h"

#include "xfsl_ioctrl_fsl.h"    /* This file is created when generating libraries */
#include "fsl_ioctrl.h"

#define IOADDR_BITS		2

#define RSHIFT_WORD(x)	((x) >> (32 - XFSL_IOCTRL_DATA_BITS))
#define LSHIFT_WORD(x)	((x) << (32 - XFSL_IOCTRL_DATA_BITS))		//Microblaze need the value in MSBits
#define	LSHIFT_CMD(x)	((x) << (32 - IOADDR_BITS))
#define CMD_MASK		(0xffL << (32 - IOADDR_BITS))

#define	CLR_OUT_CMD	0L
#define SET_OUT_CMD	1L
#define	WR_OUT_CMD	2L
#define	TG_OUT_CMD	3L

#define	RD_OUT_CMD			0L
#define RD_IN_CMD			1L
#define	RD_IN_WR_OUT_CMD	2L
#define	RD_IN_TG_OUT_CMD	3L


//Write Ops
void Xfsl_IOctrl_WriteOutputs(Xuint32 Mask)
{
	XASSERT_VOID( (LSHIFT_WORD(Mask) & CMD_MASK) == 0 );

	WRITE_FSL_IOCTRL( LSHIFT_CMD(WR_OUT_CMD) | LSHIFT_WORD(Mask) );
}

void Xfsl_IOctrl_SetOutputs(Xuint32 Mask)
{
    XASSERT_VOID( (LSHIFT_WORD(Mask) & CMD_MASK) == 0 );

	WRITE_FSL_IOCTRL( LSHIFT_CMD(SET_OUT_CMD) | LSHIFT_WORD(Mask) );
}

void Xfsl_IOctrl_ClearOutputs(Xuint32 Mask)
{
    XASSERT_VOID( (LSHIFT_WORD(Mask) & CMD_MASK) == 0 );

	WRITE_FSL_IOCTRL( LSHIFT_CMD(CLR_OUT_CMD) | LSHIFT_WORD(Mask) );
}

void Xfsl_IOctrl_ToggleOutputs(Xuint32 Mask)
{
    XASSERT_VOID( (LSHIFT_WORD(Mask) & CMD_MASK) == 0 );

	WRITE_FSL_IOCTRL( LSHIFT_CMD(TG_OUT_CMD) | LSHIFT_WORD(Mask) );
}

//Read Ops
Xuint32 Xfsl_IOctrl_ReadInputs(void)
{
	Xuint32 result;

	WRITE_CNTL_FSL_IOCTRL( LSHIFT_CMD(RD_IN_CMD) );
	READ_FSL_IOCTRL( result );

	XASSERT_NONVOID( (result & CMD_MASK) == LSHIFT_CMD(RD_IN_CMD) );

	return RSHIFT_WORD(result & ~CMD_MASK);
}

Xuint32 Xfsl_IOctrl_ReadOutputs(void)
{
	Xuint32 result;

	WRITE_CNTL_FSL_IOCTRL( LSHIFT_CMD(RD_OUT_CMD) );
	READ_FSL_IOCTRL( result );

	XASSERT_NONVOID( (result & CMD_MASK) == LSHIFT_CMD(RD_OUT_CMD) );

	return RSHIFT_WORD(result & ~CMD_MASK);
}


//Read+Write Ops
Xuint32 Xfsl_IOctrl_ReadInputsWriteOutputs(Xuint32 Mask)
{
	Xuint32 result;

    XASSERT_NONVOID( (LSHIFT_WORD(Mask) & CMD_MASK) == 0 );

	WRITE_CNTL_FSL_IOCTRL( LSHIFT_CMD(RD_IN_WR_OUT_CMD) | LSHIFT_WORD(Mask) );
	READ_FSL_IOCTRL( result );

	XASSERT_NONVOID( (result & CMD_MASK) == LSHIFT_CMD(RD_IN_WR_OUT_CMD) );

	return RSHIFT_WORD(result & ~CMD_MASK);
}

Xuint32 Xfsl_IOctrl_ReadInputsToggleOutputs(Xuint32 Mask)
{
	Xuint32 result;

    XASSERT_NONVOID( (LSHIFT_WORD(Mask) & CMD_MASK) == 0 );

	WRITE_CNTL_FSL_IOCTRL( LSHIFT_CMD(RD_IN_TG_OUT_CMD) | LSHIFT_WORD(Mask) );
	READ_FSL_IOCTRL( result );

	XASSERT_NONVOID( (result & CMD_MASK) == LSHIFT_CMD(RD_IN_TG_OUT_CMD) );

	return RSHIFT_WORD(result & ~CMD_MASK);
}
